@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2010 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -110,7 +110,7 @@ static uint64_t getu64(uint8_t *data)
 #define gets64(x) ((int64_t)getu64(x))
 
 /* Important: regval must already have been adjusted for rex extensions */
-static enum reg_enum whichreg(int32_t regflags, int regval, int rex)
+static enum reg_enum whichreg(opflags_t regflags, int regval, int rex)
 {
     if (!(regflags & (REGISTER|REGMEM)))
 	return 0;		/* Registers not permissible?! */
@@ -674,47 +674,22 @@ static int matches(const struct itemplate *t, uint8_t *data,
 	    break;
 	}
 
-	case4(0260):
-	{
-	    int vexm   = *r++;
-	    int vexwlp = *r++;
-	    ins->rex |= REX_V;
-	    if ((prefix->rex & (REX_V|REX_D|REX_P)) != REX_V)
-		return false;
-
-	    if ((vexm & 0x1f) != prefix->vex_m)
-		return false;
-
-	    switch (vexwlp & 030) {
-	    case 000:
-		if (prefix->rex & REX_W)
-		    return false;
-		break;
-	    case 010:
-		if (!(prefix->rex & REX_W))
-		    return false;
-		ins->rex &= ~REX_W;
-		break;
-	    case 020:		/* VEX.W is a don't care */
-		ins->rex &= ~REX_W;
-		break;
-	    case 030:
-		break;
+	case4(0250):
+	    if (s_field_for == op1) {
+		opx->offset = gets8(data);
+		data++;
+	    } else {
+		opx->offset = gets32(data);
+		data += 4;
 	    }
-
-	    if ((vexwlp & 007) != prefix->vex_lp)
-		return false;
-
-	    opx->segment |= SEG_RMREG;
-	    opx->basereg = prefix->vex_v;
-	    vex_ok = true;
 	    break;
-	}
 
+	case4(0260):
 	case 0270:
 	{
 	    int vexm   = *r++;
 	    int vexwlp = *r++;
+
 	    ins->rex |= REX_V;
 	    if ((prefix->rex & (REX_V|REX_D|REX_P)) != REX_V)
 		return false;
@@ -722,25 +697,34 @@ static int matches(const struct itemplate *t, uint8_t *data,
 	    if ((vexm & 0x1f) != prefix->vex_m)
 		return false;
 
-	    switch (vexwlp & 030) {
+	    switch (vexwlp & 060) {
 	    case 000:
-		if (ins->rex & REX_W)
+		if (prefix->rex & REX_W)
 		    return false;
 		break;
-	    case 010:
-		if (!(ins->rex & REX_W))
+	    case 020:
+		if (!(prefix->rex & REX_W))
 		    return false;
+		ins->rex &= ~REX_W;
+		break;
+	    case 040:		/* VEX.W is a don't care */
+		ins->rex &= ~REX_W;
+		break;
+	    case 060:
 		break;
-	    default:
-		break;		/* Need to do anything special here? */
 	    }
 
-	    if ((vexwlp & 007) != prefix->vex_lp)
-		return false;
-
-	    if (prefix->vex_v != 0)
+	    /* The 010 bit of vexwlp is set if VEX.L is ignored */
+	    if ((vexwlp ^ prefix->vex_lp) & ((vexwlp & 010) ? 03 : 07))
 		return false;
 
+	    if (c == 0270) {
+		if (prefix->vex_v != 0)
+		    return false;
+	    } else {
+		opx->segment |= SEG_RMREG;
+		opx->basereg = prefix->vex_v;
+	    }
 	    vex_ok = true;
 	    break;
 	}
@@ -821,7 +805,7 @@ static int matches(const struct itemplate *t, uint8_t *data,
 	    break;
 
 	case 0324:
-	    if (!(ins->rex & (REX_P|REX_W)) || osize != 64)
+	    if (osize != 64)
 		return false;
 	    o_used = true;
 	    break;
@@ -1097,7 +1081,7 @@ int32_t disasm(uint8_t *data, char *output, int outbufsize, int segsize,
 		    prefix.vex_lp = prefix.vex[1] & 7;
 		}
 
-		ix = itable_vex[RV_VEX][prefix.vex_m][prefix.vex_lp];
+		ix = itable_vex[RV_VEX][prefix.vex_m][prefix.vex_lp & 3];
 	    }
 	    end_prefix = true;
 	    break;
@@ -1180,7 +1164,7 @@ int32_t disasm(uint8_t *data, char *output, int outbufsize, int segsize,
 			/* If it's a mem-only EA but we have a
 			   register, die. */
 			((tmp_ins.oprs[i].segment & SEG_RMREG) &&
-			 !(MEMORY & ~(*p)->opd[i])) ||
+			 is_class(MEMORY, (*p)->opd[i])) ||
 			/* If it's a reg-only EA but we have a memory
 			   ref, die. */
 			(!(tmp_ins.oprs[i].segment & SEG_RMREG) &&
@@ -1343,7 +1327,7 @@ int32_t disasm(uint8_t *data, char *output, int outbufsize, int segsize,
 			  o->disp_size == 32 ? "dword " :
 			  o->disp_size == 16 ? "word " : ""), offs);
             segover = NULL;
-        } else if (!(REGMEM & ~t)) {
+        } else if (is_class(REGMEM, t)) {
             int started = false;
             if (t & BITS8)
                 slen +=
@@ -1553,7 +1537,7 @@ int32_t eatbyte(uint8_t *data, char *output, int outbufsize, int segsize)
     }
 
     if (str)
-	strcpy(output, str);
+	snprintf(output, outbufsize, "%s", str);
 
     return 1;
 }
